package com.jpattern.orm.generator.reflection;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import com.jpattern.orm.classmapper.IColumn;
import com.jpattern.orm.generator.ResultSetMethodHelper;
import com.jpattern.orm.generator.wrapper.TypeWrapper;

/**
 * 
 * @author Francesco Cina'
 *
 * Apr 1, 2012
 */
public class ReflectionManipulatorFactory {

	private final ResultSetMethodHelper resultSetMethodHelper;

	public ReflectionManipulatorFactory(final ResultSetMethodHelper resultSetMethodHelper) {
		this.resultSetMethodHelper = resultSetMethodHelper;
	}

	public AGetFieldManipulator getGetFieldManipulator(final IColumn column) throws SecurityException, NoSuchMethodException {
		if ( column.getGetter() == null) {
			return this.getFieldGetFieldManipulator(column);
		}
		return this.getGetterGetFieldManipulator(column);
	}

	private AGetFieldManipulator getFieldGetFieldManipulator(final IColumn column) throws SecurityException, NoSuchMethodException {
		final TypeWrapper<Object, Object> wrapper = this.getWrapper(column);
		if ( wrapper != null ) {
			return new FieldWrappedGetFieldManipulator(column.getField(), wrapper);
		}
		return new FieldGetFieldManipulator(column.getField());
	}

	private AGetFieldManipulator getGetterGetFieldManipulator(final IColumn column) throws SecurityException, NoSuchMethodException {
		final TypeWrapper<Object, Object> wrapper = this.getWrapper(column);
		if ( wrapper != null ) {
			return new GetterWrappedGetFieldManipulator(column.getGetter(), wrapper);
		}
		return new GetterGetFieldManipulator(column.getGetter());
	}


	public ASetFieldManipulator getSetFieldManipulator(final IColumn column) throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, NoSuchFieldException {
		final Field field = column.getField();
		final Method resultSetGetterMethodString = this.resultSetMethodHelper.findResultSetGetterForString(field.getType());
		final Method resultSetGetterMethodInt = this.resultSetMethodHelper.findResultSetGetterForInt(field.getType());
		if ( column.getSetter() == null) {
			return this.getFieldSetFieldManipulator(column, resultSetGetterMethodString, resultSetGetterMethodInt);
		}
		return this.getSetterSetFieldManipulator(column, resultSetGetterMethodString, resultSetGetterMethodInt);
	}

	private ASetFieldManipulator getFieldSetFieldManipulator(final IColumn column, final Method resultSetGetterMethodString, final Method resultSetGetterMethodInt) throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, NoSuchFieldException {
		final TypeWrapper<Object, Object> wrapper = this.getWrapper(column);
		if ( wrapper != null ) {
			return new FieldWrappedSetFieldManipulator(column.getField(), resultSetGetterMethodString, resultSetGetterMethodInt, wrapper);
		}
		return new FieldSetFieldManipulator(column.getField(), resultSetGetterMethodString, resultSetGetterMethodInt);
	}

	private ASetFieldManipulator getSetterSetFieldManipulator(final IColumn column, final Method resultSetGetterMethodString, final Method resultSetGetterMethodInt) throws SecurityException, NoSuchMethodException {
		final TypeWrapper<Object, Object> wrapper = this.getWrapper(column);
		if ( wrapper != null ) {
			return new SetterWrappedSetFieldManipulator(column.getSetter(), resultSetGetterMethodString, resultSetGetterMethodInt, wrapper);
		}
		return new SetterSetFieldManipulator(column.getSetter(), resultSetGetterMethodString, resultSetGetterMethodInt);
	}

	public VersionManipulator getVersionManipulator(final IColumn column, final FieldManipulator fieldManipulator) throws SecurityException, NoSuchMethodException {
		final Class<?> rsType = this.resultSetMethodHelper.findResultSetGetterForString(column.getField().getType()).getReturnType();
		return new ReflectionVersionManipulator(rsType, fieldManipulator);
	}

	public GeneratorManipulator getGeneratorManipulator(final IColumn column, final FieldManipulator fieldManipulator, final long[] validValues) throws SecurityException, NoSuchMethodException {
		final Class<?> rsType = this.resultSetMethodHelper.findResultSetGetterForString(column.getField().getType()).getReturnType();
		return new ReflectionGeneratorManipulator(rsType, fieldManipulator, validValues);
	}

	@SuppressWarnings("unchecked")
	private TypeWrapper<Object, Object> getWrapper(final IColumn column) throws SecurityException, NoSuchMethodException {
		if ( this.resultSetMethodHelper.isWrappedType(column.getField().getType()) ) {
			return (TypeWrapper<Object, Object>) this.resultSetMethodHelper.getWrapper(column.getField().getType());
		}
		return null;
	}
}
