package com.jpattern.orm.session;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import javax.sql.DataSource;

/**
 * A decorator for all the datasources which don't have an option to
 * set the default autocommit value (e.g. a datasource retrieved as a jndi resource)
 * 
 * @author Francesco Cina'
 *
 * 26/ago/2011
 */
public class AutoCommitDataSourceDecorator implements DataSource {

	private final DataSource dataSource;
	private final boolean autoCommit;

	public AutoCommitDataSourceDecorator(DataSource dataSource, boolean autoCommit) {
		this.dataSource = dataSource;
		this.autoCommit = autoCommit;
	}

	@Override
	public Connection getConnection() throws SQLException {
		final Connection connection = dataSource.getConnection();
		connection.setAutoCommit(autoCommit);
		return connection;
	}

	@Override
	public Connection getConnection(String username, String password) throws SQLException {
		final Connection connection = dataSource.getConnection(username, password);
		connection.setAutoCommit(autoCommit);
		return connection;
	}

	@Override
	public PrintWriter getLogWriter() throws SQLException {
		return dataSource.getLogWriter();
	}

	@Override
	public int getLoginTimeout() throws SQLException {
		return dataSource.getLoginTimeout();
	}

	@Override
	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		return dataSource.isWrapperFor(iface);
	}

	@Override
	public void setLogWriter(PrintWriter out) throws SQLException {
		dataSource.setLogWriter(out);
	}

	@Override
	public void setLoginTimeout(int seconds) throws SQLException {
		dataSource.setLoginTimeout(seconds);
	}

	@Override
	public <T> T unwrap(Class<T> iface) throws SQLException {
		return dataSource.unwrap(iface);
	}

}
